from typing import List
from selectolax.parser import HTMLParser

from torrra.types import Magnet, Torrent
from torrra.utils.html import parse_html
from abc import ABC, abstractmethod


class BaseIndexer(ABC):
    def _get_parser(self, url: str) -> HTMLParser:
        html = parse_html(url)
        return HTMLParser(html)

    @abstractmethod
    def search(self, query: str) -> List[Torrent]:
        pass

    @abstractmethod
    def get_magnets(self, link: str) -> List[Magnet]:
        pass



from typing import List

from torrra.indexers.base import BaseIndexer
from torrra.types import Magnet, Torrent


class Indexer(BaseIndexer):
    def search(self, query: str) -> List[Torrent]:
        url = f"https://yts.mx/browse-movies/{query}/all/all/0/latest/0/all"
        parser = self._get_parser(url)
        res = []

        nodes = parser.css(
            "div.browse-content div.browse-movie-wrap div.browse-movie-bottom"
        )
        for node in nodes:
            title_node = node.css_first("a.browse-movie-title")
            year_node = node.css_first("div.browse-movie-year")

            title = title_node.text(strip=True) if title_node else ""
            link = title_node.attributes.get("href") if title_node else ""

            if query not in title.lower() or link is None:
                continue

            year = year_node.text(strip=True) if year_node else ""

            torrent_title = f"{title} {year}".strip()
            res.append(Torrent(title=torrent_title, link=link))
        return res

    def get_magnets(self, link: str) -> List[Magnet]:
        parser = self._get_parser(link)
        res = []

        nodes = parser.css("div.modal div.modal-torrent")
        for node in nodes:
            resolution_node = node.css_first("div.modal-quality span")
            resolution = resolution_node.text(strip=True) if resolution_node else ""

            quality_info = resolution

            quality_size_nodes = node.css("p.quality-size")
            if len(quality_size_nodes) >= 2:
                source = quality_size_nodes[0].text(strip=True)
                size = quality_size_nodes[1].text(strip=True)
                quality_info = f"{source} {resolution} {size}"

            magnet_node = node.css_first("a.magnet")
            magnet_uri = magnet_node.attributes.get("href") if magnet_node else ""

            if not magnet_uri:
                continue

            res.append(Magnet(title=quality_info, magnet_uri=magnet_uri))
        return res



import importlib
from prompt_toolkit.shortcuts import CompleteStyle
import questionary
from questionary import Choice
from typing import List
from rich.console import Console

from torrra.downloader import download_magnet
from torrra.indexers import INDEXERS
from torrra.types import Magnet, Torrent

console = Console()


def main() -> None:
    query = questionary.text("Search:").ask()

    if not query:
        console.print("[red]No query entered. Exiting...[/red]")
        return

    indexer_name = questionary.select(
        "Choose an indexer:", choices=list(INDEXERS.keys())
    ).ask()
    indexer_module_path = INDEXERS[indexer_name]
    indexer = importlib.import_module(indexer_module_path).Indexer()

    with console.status(
        f"[bold green]Searching {indexer_name} for '{query}'...[/bold green]"
    ):
        torrents: List[Torrent] = indexer.search(query)

    if not torrents:
        console.print("[yellow]Could not find any torrents. Exiting...[/yellow]")
        return

    torrent_choices = [
        Choice(title=torrent.title, value=torrent) for torrent in torrents
    ]
    selected_torrent: Torrent | None = questionary.select(
        "Select a torrent:", choices=torrent_choices
    ).ask()

    if not selected_torrent:
        console.print("[yellow]No torrent selected. Exiting...[/yellow]")
        return

    with console.status(
        f"[bold green]Fetching magnet links for '{selected_torrent.title}'...[/bold green]"
    ):
        magnets: List[Magnet] = indexer.get_magnets(selected_torrent.link)

    if not magnets:
        console.print("[yellow]No magnet links found. Exiting...[/yellow]")
        return

    magnet_choices = [Choice(title=magnet.title, value=magnet) for magnet in magnets]
    selected_magnet: Magnet | None = questionary.select(
        "Select:", choices=magnet_choices
    ).ask()

    if not selected_magnet:
        console.print("[yellow]No magnet selected. Exiting...[/yellow]")
        return

    save_path = questionary.path(
        "Download path:", only_directories=True, complete_style=CompleteStyle.COLUMN
    ).ask()
    download_magnet(selected_magnet.magnet_uri, save_path)



import sys
try:
    import libtorrent as lt
except ImportError:
    print("Error: libtorrent not found")
    print("Installation guide: https://github.com/stabldev/torrra?tab=readme-ov-file#-installation")
    sys.exit(1)

import time
from rich.progress import (
    Progress,
    BarColumn,
    TransferSpeedColumn,
    TimeRemainingColumn,
    TextColumn,
    TaskProgressColumn,
    DownloadColumn,
)
from rich.console import Console

console = Console()


def download_magnet(magnet_uri: str, save_path: str):
    ses = lt.session()
    ses.listen_on(6881, 6891)

    params = {
        "save_path": save_path,
        "storage_mode": lt.storage_mode_t(2),
    }

    handle = lt.add_magnet_uri(ses, magnet_uri, params)

    console.print("[bold green]Downloading metadata...[/bold green]")
    while not handle.has_metadata():
        time.sleep(0.5)

    torrent_info = handle.get_torrent_info()
    total_size = torrent_info.total_size()
    file_count = torrent_info.num_files()

    console.print(
        f"[bold green]Starting download: [cyan]{handle.name()}[/cyan][/bold green]"
    )
    console.print(
        f"[yellow]Size:[/yellow] {total_size/(1024*1024):.2f} MB | [yellow]Files:[/yellow] {file_count}"
    )

    with Progress(
        TextColumn("[bold blue]{task.fields[filename]}"),
        BarColumn(),
        TaskProgressColumn(),
        DownloadColumn(),
        TransferSpeedColumn(),
        TimeRemainingColumn(),
        console=console,
    ) as progress:
        task = progress.add_task(
            "download", filename=handle.name(), peers=0, total=total_size
        )

        prev_downloaded = 0
        prev_time = time.time()

        while not handle.is_seed():
            s = handle.status()

            # calculate real-time download speed
            cur_time = time.time()
            downloaded = s.total_done
            elapsed = cur_time - prev_time
            speed = (downloaded - prev_downloaded) / elapsed if elapsed > 0 else 0

            progress.update(task, completed=downloaded, speed=speed)

            prev_downloaded = downloaded
            prev_time = cur_time
            time.sleep(0.5)

    console.print(
        f"\n[bold green]✓ Download complete: [cyan]{handle.name()}[/cyan][/bold green]"
    )
    console.print(f"[green]Saved to: [underline]{save_path}[/underline][/green]")
